@@ -68,10 +68,10 @@
 # and doneCallback:
 #
 # errorNotifyCallback, if provided, will be used to print out any error
-# messages (e.g. song file not found). This allows the module to fit 
+# messages (e.g. song file not found). This allows the module to fit
 # together well with GUI playlist managers by utilising the same GUI's
 # error popup window mechanism (or similar). If no callback is provided,
-# errors are printed to stdout. errorNotifyCallback should take one 
+# errors are printed to stdout. errorNotifyCallback should take one
 # parameter, the error string, e.g.:
 #   def errorPopup (ErrorString):
 #       msgBox (ErrorString)
@@ -129,10 +129,6 @@
 # screen is updated, but during normal CD+G operation only a small
 # number of segments are likely to be changed at update time.
 #
-# NOTE: Pygame does not currently support querying the length
-# of an MP3 track, therefore the GetLength() method is not
-# currently implemented.
-#
 # Here follows a description of the important data stored by
 # the class:
 #
@@ -150,21 +146,21 @@
 # CdgPacketReader.__cdgPixelColours[300][216]
 # Store the colour index for every single pixel. The values stored
 # are indeces into our colour table, rather than actual pygame
-# colour representations. It's unfortunate that we need to store 
+# colour representations. It's unfortunate that we need to store
 # all this data, when in fact the pixel colour is available from
 # cdgSurfarray, but we need it for the Tile Block XOR command.
 # The XOR command performs an XOR of the colour index currently
 # at the pixel, with the new colour index. We therefore need to
-# know the actual colour index at that pixel - we can't do a 
+# know the actual colour index at that pixel - we can't do a
 # get_at() on the screen, or look in cdgSurfarray, and map the RGB
-# colour back to a colour index because some CDG files have the 
+# colour back to a colour index because some CDG files have the
 # same colour in two places in the table, making it impossible to
 # determine which index is relevant for the XOR.
 #
-# CdgPacketReader.__cdgPresetColourIndex 
+# CdgPacketReader.__cdgPresetColourIndex
 # Preset Colour (index into colour table)
 #
-# CdgPacketReader.__cdgPresetColourIndex 
+# CdgPacketReader.__cdgPresetColourIndex
 # Border Colour (index into colour table)
 #
 # CdgPacketReader.__updatedTiles
@@ -183,10 +179,14 @@ import sys, pygame, os, string, math
 
 # Import the optimised C version if available, or fall back to Python
 try:
-    import _pycdgAux as aux
+    import _pycdgAux as aux_c
 except ImportError:
-    print "Using Python implementation of CDG interpreter."
-    import pycdgAux as aux
+    aux_c = None
+
+try:
+    import pycdgAux as aux_python
+except ImportError:
+    aux_python = None
 
 CDG_DISPLAY_WIDTH   = 288
 CDG_DISPLAY_HEIGHT  = 192
@@ -206,11 +206,12 @@ TILE_HEIGHT             = CDG_DISPLAY_HEIGHT / TILES_PER_COL
 # cdgPlayer Class
 class cdgPlayer(pykPlayer):
     # Initialise the player instace
-    def __init__(self, song, errorNotifyCallback=None, doneCallback=None):
+    def __init__(self, song, songDb, errorNotifyCallback=None,
+                 doneCallback=None):
         """The first parameter, song, may be either a pykdb.SongStruct
         instance, or it may be a filename. """
 
-        pykPlayer.__init__(self, song, errorNotifyCallback, doneCallback)
+        pykPlayer.__init__(self, song, songDb, errorNotifyCallback, doneCallback)
 
         # With the nomusic option no music will be played.
         soundFileData = None
@@ -244,30 +245,41 @@ class cdgPlayer(pykPlayer):
 
         manager.InitPlayer(self)
         manager.OpenDisplay()
-        manager.display.fill((0, 0, 0))
+        manager.surface.fill((0, 0, 0))
 
         # A working surface for blitting tiles, one at a time.
         self.workingTile = pygame.Surface((TILE_WIDTH, TILE_HEIGHT),
-                                          0, manager.display)
+                                          0, manager.surface)
 
         # A surface that contains the set of all tiles as they are to
         # be assembled onscreen.  This surface is kept at the original
         # scale, then zoomed to display size.  It is only used if
-        # options.zoom_mode == 'soft'.
+        # settings.CdgZoom == 'soft'.
         self.workingSurface = pygame.Surface((CDG_DISPLAY_WIDTH, CDG_DISPLAY_HEIGHT),
                                              pygame.HWSURFACE,
-                                             manager.display)
+                                             manager.surface)
 
         self.borderColour = None
         self.computeDisplaySize()
 
+        aux = aux_c
+        if not aux or not manager.settings.CdgUseC:
+            print "Using Python implementation of CDG interpreter."
+            aux = aux_python
+
         # Open the cdg and sound files
         self.packetReader = aux.CdgPacketReader(self.cdgFileData.GetData(), self.workingTile)
-        
+        manager.setCpuSpeed('cdg')
+
         if self.soundFileData:
-            audioProperties = self.getAudioProperties(self.soundFileData)
+            # Play the music normally.
+            audioProperties = None
+            if manager.settings.UseMp3Settings:
+                audioProperties = self.getAudioProperties(self.soundFileData)
+            if audioProperties == None:
+                audioProperties = (None, None, None)
             try:
-                manager.OpenAudio(suggestedProperties = (audioProperties))
+                manager.OpenAudio(*audioProperties)
                 pygame.mixer.music.load(self.soundFileData.GetFilepath())
             except:
                 self.Close()
@@ -282,16 +294,18 @@ class cdgPlayer(pykPlayer):
             # be the right amount of delay, but it should usually be
             # pretty close.
             self.InternalOffsetTime = -manager.GetAudioBufferMS()
+
         else:
+            # Don't play anything.
             self.InternalOffsetTime = 0
-            
+
         # Set the CDG file at the beginning
         self.cdgReadPackets = 0
         self.cdgPacketsDue = 0
         self.LastPos = self.curr_pos = 0
 
         # Some session-wide constants.
-        self.ms_per_update = (1000.0 / manager.options.fps)        
+        self.ms_per_update = (1000.0 / manager.options.fps)
 
     def doPlay(self):
         if self.soundFileData:
@@ -324,6 +338,10 @@ class cdgPlayer(pykPlayer):
             pygame.mixer.music.rewind()
             pygame.mixer.music.stop()
 
+    def GetLength(self):
+        """Give the number of seconds in the song."""
+        return self.soundLength
+
     # Get the current time (in milliseconds). Blocks if pygame is
     # not initialised yet.
     def GetPos(self):
@@ -341,30 +359,38 @@ class cdgPlayer(pykPlayer):
 
         # Remove irrelevant options.
         parser.remove_option('--font-scale')
-        
+
         return parser
 
 
     def shutdown(self):
         # This will be called by the pykManager to shut down the thing
         # immediately.
-        pygame.mixer.music.stop()
+        if self.soundFileData:
+            if manager.audioProps:
+                pygame.mixer.music.stop()
+
+        # Make sure our surfaces are deallocated before we call up to
+        # CloseDisplay(), otherwise bad things can happen.
+        self.workingSurface = None
+        self.workingTile = None
+        self.packetReader = None
         pykPlayer.shutdown(self)
 
     def doStuff(self):
         pykPlayer.doStuff(self)
-        
+
         # Check whether the songfile has moved on, if so
         # get the relevant CDG data and update the screen.
-        if self.State == STATE_PLAYING:
-            self.curr_pos = self.GetPos() + self.InternalOffsetTime + manager.UserOffsetTime - self.pauseOffsetTime
+        if self.State == STATE_PLAYING or self.State == STATE_CAPTURING:
+            self.curr_pos = self.GetPos() + self.InternalOffsetTime + manager.settings.SyncDelayMs - self.pauseOffsetTime
 
             self.cdgPacketsDue = int((self.curr_pos * 300) / 1000)
             numPackets = self.cdgPacketsDue - self.cdgReadPackets
             if numPackets > 0:
                 if not self.packetReader.DoPackets(numPackets):
                     # End of file.
-                    print "End of file on cdg."
+                    #print "End of file on cdg."
                     self.Close()
                 self.cdgReadPackets += numPackets
 
@@ -378,9 +404,9 @@ class cdgPlayer(pykPlayer):
 
     def doResize(self, newSize):
         self.computeDisplaySize()
-    
+
         if self.borderColour != None:
-            manager.display.fill(self.borderColour)
+            manager.surface.fill(self.borderColour)
 
         self.packetReader.MarkTilesDirty()
 
@@ -388,16 +414,16 @@ class cdgPlayer(pykPlayer):
         """ Figures out what scale and placement to use for blitting
         tiles to the screen.  This must be called at startup, and
         whenever the window size changes. """
-        
+
         winWidth, winHeight = manager.displaySize
 
         # Compute an appropriate uniform scale to letterbox the image
         # within the window
         scale = min(float(winWidth) / CDG_DISPLAY_WIDTH,
                     float(winHeight) / CDG_DISPLAY_HEIGHT)
-        if manager.options.zoom_mode == 'none':
+        if manager.settings.CdgZoom == 'none':
             scale = 1
-        elif manager.options.zoom_mode == 'int':
+        elif manager.settings.CdgZoom == 'int':
             if scale < 1:
                 scale = 1.0/math.ceil(1.0/scale)
             else:
@@ -407,7 +433,7 @@ class cdgPlayer(pykPlayer):
         scaledWidth = int(scale * CDG_DISPLAY_WIDTH)
         scaledHeight = int(scale * CDG_DISPLAY_HEIGHT)
 
-        if manager.options.zoom_mode == 'full':
+        if manager.settings.CdgZoom == 'full':
             # If we are allowing non-proportional scaling, allow
             # scaledWidth and scaledHeight to be independent.
             scaledWidth = winWidth
@@ -418,7 +444,7 @@ class cdgPlayer(pykPlayer):
         self.displayColOffset = (winHeight - scaledHeight) / 2
 
         # Calculate the scaled width and height for each tile
-        if manager.options.zoom_mode == 'soft':
+        if manager.settings.CdgZoom == 'soft':
             self.displayTileWidth = CDG_DISPLAY_WIDTH / TILES_PER_ROW
             self.displayTileHeight = CDG_DISPLAY_HEIGHT / TILES_PER_COL
         else:
@@ -440,12 +466,6 @@ class cdgPlayer(pykPlayer):
         if soundFileData.Ext == '.mp3':
             audioProperties = self.getMp3AudioProperties(soundFileData)
 
-        if audioProperties == None:
-            # We don't know how to determine the audio properties from
-            # this file.  Punt; almost all CD rips will be the
-            # following properties.
-            audioProperties = (44100, -16, 2)
-
         return audioProperties
 
     def getMp3AudioProperties(self, soundFileData):
@@ -456,10 +476,13 @@ class cdgPlayer(pykPlayer):
         try:
             import MP3Info
         except:
+            print "Failed to load MP3Info, will not be able to determine extra MP3 information."
+            self.soundLength = 0
             return None
 
         import cStringIO
         m = MP3Info.MPEG(cStringIO.StringIO(soundFileData.GetData()))
+        self.soundLength = m.length
 
         channels = 1
         if 'stereo' in m.mode:
@@ -483,48 +506,47 @@ class cdgPlayer(pykPlayer):
         # There are four different approaches for blitting tiles onto
         # the display:
 
-        # options.zoom_mode == 'none':
+        # settings.CdgZoom == 'none':
         #   No scaling.  The CDG graphics are centered within the
         #   display.  When a tile is dirty, it is blitted directly to
-        #   manager.display.  After all dirty tiles have been blitted,
+        #   manager.surface.  After all dirty tiles have been blitted,
         #   we then use display.update to flip only those rectangles
         #   on the screen that have been blitted.
 
-        # options.zoom_mode = 'quick':
+        # settings.CdgZoom = 'quick':
         #   Trivial scaling.  Similar to 'none', but each tile is
         #   first scaled to its target scale using
         #   pygame.transform.scale(), which is quick but gives a
         #   pixelly result.  The scaled tile is then blitted to
-        #   manager.display.
+        #   manager.surface.
 
-        # options.zoom_mode = 'int':
+        # settings.CdgZoom = 'int':
         #   The same as 'quick', but the scaling is constrained to be
         #   an integer multiple or divisor of its original size, which
         #   may reduce artifacts somewhat.
 
-        # options.zoom_mode = 'full':
+        # settings.CdgZoom = 'full':
         #   The same as 'quick', but the scaling is allowed to
         #   completely fill the window in both x and y, regardless of
         #   aspect ratio constraints.
 
-        # options.zoom_mode = 'soft':
+        # settings.CdgZoom = 'soft':
         #   Antialiased scaling.  We blit all tiles onto
         #   self.workingSurface, which is maintained as the non-scaled
         #   version of the CDG graphics, similar to 'none'.  Then,
         #   after all dirty tiles have been blitted to
         #   self.workingSurface, we use pygame.transform.rotozoom() to
         #   make a nice, antialiased scaling of workingSurface to
-        #   manager.display, and then flip the whole display.  (We
+        #   manager.surface, and then flip the whole display.  (We
         #   can't scale and blit the tiles one a time in this mode,
         #   since that introduces artifacts between the tile edges.)
-
         borderColour = self.packetReader.GetBorderColour()
         if borderColour != self.borderColour:
             # When the border colour changes, blit the whole screen
             # and redraw it.
             self.borderColour = borderColour
             if borderColour != None:
-                manager.display.fill(borderColour)
+                manager.surface.fill(borderColour)
                 self.packetReader.MarkTilesDirty()
 
         dirtyTiles = self.packetReader.GetDirtyTiles()
@@ -539,15 +561,15 @@ class cdgPlayer(pykPlayer):
         for row, col in dirtyTiles:
             self.packetReader.FillTile(self.workingTile, row, col)
 
-            if manager.options.zoom_mode == 'none':
+            if manager.settings.CdgZoom == 'none':
                 # The no-scale approach.
                 rect = pygame.Rect(self.displayTileWidth * row + self.displayRowOffset,
                                    self.displayTileHeight * col + self.displayColOffset,
                                    self.displayTileWidth, self.displayTileHeight)
-                manager.display.blit(self.workingTile, rect)
+                manager.surface.blit(self.workingTile, rect)
                 rect_list.append(rect)
 
-            elif manager.options.zoom_mode == 'soft':
+            elif manager.settings.CdgZoom == 'soft':
                 # The soft-scale approach.
                 self.workingSurface.blit(self.workingTile, (self.displayTileWidth * row, self.displayTileHeight * col))
 
@@ -557,26 +579,27 @@ class cdgPlayer(pykPlayer):
                 rect = pygame.Rect(self.displayTileWidth * row + self.displayRowOffset,
                                    self.displayTileHeight * col + self.displayColOffset,
                                    self.displayTileWidth, self.displayTileHeight)
-                manager.display.blit(scaled, rect)
+                manager.surface.blit(scaled, rect)
                 rect_list.append(rect)
 
-        if manager.options.zoom_mode == 'soft':
+        if manager.settings.CdgZoom == 'soft':
             # Now scale and blit the whole screen.
             scaled = pygame.transform.rotozoom(self.workingSurface, 0, self.displayScale)
-            manager.display.blit(scaled, (self.displayRowOffset, self.displayColOffset))
-            pygame.display.flip()
+            manager.surface.blit(scaled, (self.displayRowOffset, self.displayColOffset))
+            manager.Flip()
         elif len(rect_list) < 24:
             # Only update those areas which have changed
-            pygame.display.update(rect_list)
+            if manager.display:
+                pygame.display.update(rect_list)
         else:
-            pygame.display.flip()
+            manager.Flip()
 
 def defaultErrorPrint(ErrorString):
     print (ErrorString)
 
 # Can be called from the command line with the CDG filepath as parameter
 def main():
-    player = cdgPlayer(None)
+    player = cdgPlayer(None, None)
     player.Play()
     manager.WaitForPlayer()
 
@@ -585,4 +608,4 @@ if __name__ == "__main__":
     #import profile
     #result = profile.run('main()', 'pycdg.prof')
     #sys.exit(result)
-    
+
